@@ -13,11 +13,12 @@ use utf8;
 
 our @EXPORT = qw( whois get_whois );
 
-our $VERSION = '2.13';
+our $VERSION = '2.43';
 
-our ($OMIT_MSG, $CHECK_FAIL, $CHECK_EXCEED, $CACHE_DIR, $USE_CNAMES, $TIMEOUT, $DEBUG) = (0) x 7;
+our ($OMIT_MSG, $CHECK_FAIL, $CHECK_EXCEED, $CACHE_DIR, $TIMEOUT, $DEBUG) = (0) x 7;
 our $CACHE_TIME = 60;
 our $SET_CODEPAGE = '';
+our $SILENT_MODE = 0;
 our (%notfound, %strip, @SRC_IPS, %POSTPROCESS);
 
 our $class = __PACKAGE__;
@@ -26,7 +27,7 @@ my $last_cache_clear_time;
 
 sub whois_config {
     my ($par) = @_;
-    my @parnames = qw(OMIT_MSG CHECK_FAIL CACHE_DIR CACHE_TIME USE_CNAMES TIMEOUT @SRC_IPS);
+    my @parnames = qw(OMIT_MSG CHECK_FAIL CHECK_EXCEED CACHE_DIR CACHE_TIME TIMEOUT @SRC_IPS);
     foreach my $parname (@parnames) {
         if (exists($par->{$parname})) {
 	    no strict 'refs';
@@ -57,7 +58,7 @@ sub whois {
     my $res = Net::Whois::Raw::Common::get_from_cache(
         $dom, $CACHE_DIR, $CACHE_TIME
     );
-    
+
     my ($res_text, $res_srv, $res_text2);
 
     if ($res) {
@@ -72,13 +73,15 @@ sub whois {
     else {
         ($res_text, $res_srv) = get_whois($dom, $server, $which_whois);
     }
-    
+
     $res_srv = '' if $res_srv && $res_srv eq 'www_whois';
-    
-    utf8::decode( $res_text ); # Perl whyly loss utf8 flag
-    
-    $res_text = encode( $SET_CODEPAGE, $res_text ) if $SET_CODEPAGE;
-    
+
+    if ( $which_whois ne 'QRY_ALL' ) {
+	utf8::decode( $res_text ); # Perl whyly loss utf8 flag
+
+	$res_text = encode( $SET_CODEPAGE, $res_text ) if $SET_CODEPAGE;
+    }
+
     return wantarray ? ($res_text, $res_srv) : $res_text;
 }
 
@@ -91,7 +94,7 @@ sub get_whois {
         or return undef;
 
     Net::Whois::Raw::Common::write_to_cache($dom, $whois, $CACHE_DIR);
-    
+
     if ($which_whois eq 'QRY_LAST') {
 	my $thewhois = $whois->[-1];
         return wantarray ? ($thewhois->{text}, $thewhois->{srv}) : $thewhois->{text};
@@ -108,19 +111,21 @@ sub get_whois {
 sub get_all_whois {
     my ($dom, $srv, $norecurse) = @_;
 
-    $srv ||= Net::Whois::Raw::Common::get_server( $dom, $USE_CNAMES );
+    my $is_ns = 0;
+    $is_ns = 1 if $dom =~ s/.NS$//i;
+
+    $srv ||= Net::Whois::Raw::Common::get_server( $dom, $is_ns );
 
     if ($srv eq 'www_whois') {
 	my ($responce, $ishtml) = www_whois_query( $dom );
 	return $responce ? [ { text => $responce, srv => $srv } ] : $responce;
     }
 
-    my $is_ns = 0;
-    $is_ns = 1 if $dom =~ s/.NS$//i;
-
     my @whois = recursive_whois( $dom, $srv, [], $norecurse, $is_ns );
 
-    return process_whois_answers( \@whois, $dom );
+    my $whois_answers = process_whois_answers( \@whois, $dom );
+
+    return $whois_answers;
 }
 
 sub process_whois_answers {
@@ -144,7 +149,7 @@ sub process_whois_answers {
         }
         $level++;
     }
-    
+
     return \@processed_whois;
 }
 
@@ -172,7 +177,7 @@ sub recursive_whois {
 	    $newsrv = $Net::Whois::Raw::Data::ip_whois_servers{ $1 };
     	}
 	elsif ((/OrgID:\s+(\w+)/ || /descr:\s+(\w+)/) && Net::Whois::Raw::Common::is_ipaddr($dom)) {
-	    my $val = $1;	
+	    my $val = $1;
 	    if($val =~ /^(?:RIPE|APNIC|KRNIC|LACNIC)$/) {
 		$newsrv = $Net::Whois::Raw::Data::ip_whois_servers{ $val };
 		last;
@@ -181,8 +186,8 @@ sub recursive_whois {
 	elsif (/^\s+Maintainer:\s+RIPE\b/ && Net::Whois::Raw::Common::is_ipaddr($dom)) {
             $newsrv = $Net::Whois::Raw::Data::servers{RIPE};
 	}
-	elsif ( $is_ns && $srv eq $Net::Whois::Raw::Data::servers{NS} && /No match for nameserver/ && $dom =~ /.name$/i ) {
-	    $newsrv = $Net::Whois::Raw::Data::servers{NAME};
+	elsif ( $is_ns && $srv ne $Net::Whois::Raw::Data::servers{NS} ) {
+	    $newsrv = $Net::Whois::Raw::Data::servers{NS};
 	}
     }
 
@@ -202,7 +207,7 @@ sub recursive_whois {
 	my $new_whois = scalar(@new_whois_recs) ? $new_whois_recs[0]->{text} : '';
 	my $notfound = $Net::Whois::Raw::Data::notfound{$newsrv};
 
-        if ( $new_whois && !$@ && not ( $notfound && $new_whois =~ /$notfound/m ) ) {
+        if ( $new_whois && !$@ && not ( $notfound && $new_whois =~ /$notfound/im ) ) {
             if ( $is_ns ) {
                 unshift @whois_recs, @new_whois_recs;
             }
@@ -226,10 +231,15 @@ sub whois_query {
 
     # Prepare for query
 
-    my @sockparams;
+    my (@sockparams, $sock);
+
     if ($class->can ('whois_query_sockparams')) {
         @sockparams = $class->whois_query_sockparams ($dom, $srv);
     }
+    # hook for outside defined socket
+    elsif ($class->can ('whois_query_socket')) {
+        $sock = $class->whois_query_socket ($dom, $srv);
+    }
     elsif (scalar(@SRC_IPS)) {
         my $src_ip = $SRC_IPS[0];
         push @SRC_IPS, shift @SRC_IPS; # rotate ips
@@ -241,7 +251,7 @@ sub whois_query {
 
     print "QUERY: $whoisquery; SRV: $srv, ".
 	    "OMIT_MSG: $OMIT_MSG, CHECK_FAIL: $CHECK_FAIL, CACHE_DIR: $CACHE_DIR, ".
-	    "CACHE_TIME: $CACHE_TIME, USE_CNAMES: $USE_CNAMES, TIMEOUT: $TIMEOUT\n" if $DEBUG >= 2;
+	    "CACHE_TIME: $CACHE_TIME, TIMEOUT: $TIMEOUT\n" if $DEBUG >= 2;
 
     my $prev_alarm = 0;
     my @lines;
@@ -251,28 +261,32 @@ sub whois_query {
     eval {
         local $SIG{'ALRM'} = sub { die "Connection timeout to $srv" };
         $prev_alarm = alarm $TIMEOUT if $TIMEOUT;
-        my $sock = IO::Socket::INET->new(@sockparams) || Carp::confess "$srv: $!: ".join(', ', @sockparams);
-        
+
+        unless($sock){
+            $sock = IO::Socket::INET->new(@sockparams) || die "$srv: $!: ".join(', ', @sockparams);
+        }
+
         if ($class->can ('whois_socket_fixup')) {
             my $new_sock = $class->whois_socket_fixup ($sock);
-	    $sock = $new_sock if $new_sock;
+            $sock = $new_sock if $new_sock;
         }
 
 	if ($DEBUG > 2) {
-	    _require_once('Data::Dumper');
-	    print "Socket: ".Dumper($sock);
+	    require Data::Dumper;      
+	    print "Socket: ". Data::Dumper::Dumper($sock);
 	}
 
         $sock->print( $whoisquery, "\r\n" );
         # TODO: $soc->read, parameters for read chunk size, max content length
-        while (my $str = <$sock>) {
+        # Now you can redefine SOCK_CLASS::getline method as you want
+        while (my $str = $sock->getline) {
             push @lines, $str;
         }
         $sock->close;
     };
 
     alarm $prev_alarm;
-    die $@ if $@;
+    Carp::confess $@ if $@;
 
     foreach (@lines) { s/\r//g; }
 
@@ -286,11 +300,11 @@ sub www_whois_query {
 
     my ($resp, $url);
     my ($name, $tld) = Net::Whois::Raw::Common::split_domain( $dom );
-	# my ($url, %form) = Net::Whois::Raw::Common::get_http_query_url($dom, $reserve);
+
     my $http_query_urls = Net::Whois::Raw::Common::get_http_query_url($dom);
-    
+
     foreach my $qurl ( @{$http_query_urls} ) {
-    
+
 	# load-on-demand
 	unless ($INC{'LWP/UserAgent.pm'}) {
 	    require LWP::UserAgent;
@@ -302,12 +316,21 @@ sub www_whois_query {
 	    import HTTP::Headers;
 	    import URI::URL;
 	}
-    
+
 	my $referer = delete $qurl->{form}{referer} if $qurl->{form} && defined $qurl->{form}{referer};
 	my $method = ( $qurl->{form} && scalar(keys %{$qurl->{form}}) ) ? 'POST' : 'GET';
 
-	my $ua = new LWP::UserAgent( parse_head => 0 );
+    my $ua;
+
+    # hook for outside defined lwp
+    if ($class->can ('whois_query_ua')) {
+        $ua = $class->whois_query_ua ($dom);
+    }
+
+    unless($ua){
+        $ua = new LWP::UserAgent( parse_head => 0 );
 	$ua->agent('Mozilla/5.0 (X11; U; Linux i686; ru; rv:1.9.0.5) Gecko/2008121622 Fedora/3.0.5-1.fc10 Firefox/3.0.5');
+    }
 	my $header = HTTP::Headers->new;
 	$header->header('Referer' => $referer) if $referer;
 	my $req = new HTTP::Request $method, $qurl->{url}, $header;
@@ -337,7 +360,7 @@ sub www_whois_query {
 	    last;
 	}
     }
-    
+
     return undef unless $resp;
 
     chomp $resp;
@@ -350,17 +373,6 @@ sub www_whois_query {
     return wantarray ? ($resp, $ishtml) : $resp;
 }
 
-sub _require_once ($) {
-    my ($module) = @_;
-
-    my $module_file = $module.'.pm';
-    $module_file =~ s/::/\//g;
-
-    unless ($INC{$module_file}) {
-	eval "require $module";
-	import $module;
-    }
-}
 
 sub import {
     my $mypkg = shift;
@@ -377,12 +389,12 @@ __END__
 
 =head1 NAME
 
-Net::Whois::Raw - Get Whois information for domains
+Net::Whois::Raw -- Get Whois information for domains
 
 =head1 SYNOPSIS
 
   use Net::Whois::Raw;
-  
+
   $dominfo = whois('perl.com');
   ($dominfo, $whois_server) = whois('funet.fi');
   $reginfo = whois('REGRU-REG-RIPN', 'whois.ripn.net');
@@ -415,12 +427,7 @@ Net::Whois::Raw - Get Whois information for domains
 	# Cache files will be cleared after not accessed
         # for a specific number of minutes. Documents will not be
         # cleared if they keep get requested for, independent
-        # of disk space. Default is not to clear the cache.
-
-  $Net::Whois::Raw::USE_CNAMES = 1;
-	# Use whois-servers.net to get the whois server
-        # name when possible. Default is to use the 
-        # hardcoded defaults.
+        # of disk space.
 
   $Net::Whois::Raw::TIMEOUT = 10;
 	# Cancel the request if connection is not made within
@@ -497,9 +504,56 @@ C<'QRY_ALL'> -
 
 =back
 
+=head1 USER DEFINED FUNCTIONS
+
+=over 3
+
+=item whois_query_sockparams( DOMAIN, SRV )
+
+You can set your own IO::Socket::INET params like this:
+
+    *Net::Whois::Raw::whois_query_sockparams = sub {
+        my $class  = shift;
+        my $domain = shift;
+        my $name   = shift;
+
+        return (
+            PeerAddr => $name,
+            PeerPort => 43,
+            # LocalHost => ,
+            # LocalPort =>
+        );
+    };
+
+=item whois_query_socket( DOMAIN, SRV )
+
+You can set your own IO::Socket::INET like this:
+
+    *Net::Whois::Raw::whois_query_socket = sub {
+        my $class  = shift;
+        my $domain = shift;
+        my $name   = shift;
+
+        $name .= ':43';
+        return IO::Socket::INET->new();
+    };
+
+=item whois_query_ua( DOMAIN, SRV )
+
+You can set your own LWP::UserAgent like this:
+
+    *Net::Whois::Raw::whois_query_ua = sub {
+        my $class  = shift;
+        my $domain = shift;
+
+        return LWP::UserAgent->new();
+    };
+
+=back
+
 =head1 AUTHOR
 
-Original author Ariel Brosh B<schop@cpan.org>, 
+Original author Ariel Brosh B<schop@cpan.org>,
 Inspired by jwhois.pl available on the net.
 
 Since Ariel has passed away in September 2002:
@@ -543,8 +597,8 @@ Not available anymore.
 =head1 LEGAL
 
 Notice that registrars forbid querying their whois servers as a part of
-a search engine, or querying for a lot of domains by script. 
-Also, omitting the copyright information (that was requested by users of this 
+a search engine, or querying for a lot of domains by script.
+Also, omitting the copyright information (that was requested by users of this
 module) is forbidden by the registrars.
 
 =head1 SEE ALSO
